# coding: utf-8

"""
    Channel Access Token API

    This document describes Channel Access Token API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io

from pydantic.v1 import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic.v1 import Field, StrictStr

from linebot.v3.oauth.models.channel_access_token_key_ids_response import ChannelAccessTokenKeyIdsResponse
from linebot.v3.oauth.models.issue_channel_access_token_response import IssueChannelAccessTokenResponse
from linebot.v3.oauth.models.issue_short_lived_channel_access_token_response import IssueShortLivedChannelAccessTokenResponse
from linebot.v3.oauth.models.issue_stateless_channel_access_token_response import IssueStatelessChannelAccessTokenResponse
from linebot.v3.oauth.models.verify_channel_access_token_response import VerifyChannelAccessTokenResponse

from linebot.v3.oauth.api_client import ApiClient
from linebot.v3.oauth.api_response import ApiResponse
from linebot.v3.oauth.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class ChannelAccessToken(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client
        self.line_base_path = "https://api.line.me"


    @validate_arguments
    def gets_all_valid_channel_access_token_key_ids(self, client_assertion_type : Annotated[StrictStr, Field(..., description="`urn:ietf:params:oauth:client-assertion-type:jwt-bearer`")], client_assertion : Annotated[StrictStr, Field(..., description="A JSON Web Token (JWT) (opens new window)the client needs to create and sign with the private key.")], **kwargs) -> ChannelAccessTokenKeyIdsResponse:  # noqa: E501
        """gets_all_valid_channel_access_token_key_ids  # noqa: E501

        Gets all valid channel access token key IDs.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.gets_all_valid_channel_access_token_key_ids(client_assertion_type, client_assertion, async_req=True)
        >>> result = thread.get()

        :param client_assertion_type: `urn:ietf:params:oauth:client-assertion-type:jwt-bearer` (required)
        :type client_assertion_type: str
        :param client_assertion: A JSON Web Token (JWT) (opens new window)the client needs to create and sign with the private key. (required)
        :type client_assertion: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ChannelAccessTokenKeyIdsResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the gets_all_valid_channel_access_token_key_ids_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.gets_all_valid_channel_access_token_key_ids_with_http_info(client_assertion_type, client_assertion, **kwargs)  # noqa: E501

    @validate_arguments
    def gets_all_valid_channel_access_token_key_ids_with_http_info(self, client_assertion_type : Annotated[StrictStr, Field(..., description="`urn:ietf:params:oauth:client-assertion-type:jwt-bearer`")], client_assertion : Annotated[StrictStr, Field(..., description="A JSON Web Token (JWT) (opens new window)the client needs to create and sign with the private key.")], **kwargs) -> ApiResponse:  # noqa: E501
        """gets_all_valid_channel_access_token_key_ids  # noqa: E501

        Gets all valid channel access token key IDs.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.gets_all_valid_channel_access_token_key_ids_with_http_info(client_assertion_type, client_assertion, async_req=True)
        >>> result = thread.get()

        :param client_assertion_type: `urn:ietf:params:oauth:client-assertion-type:jwt-bearer` (required)
        :type client_assertion_type: str
        :param client_assertion: A JSON Web Token (JWT) (opens new window)the client needs to create and sign with the private key. (required)
        :type client_assertion: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ChannelAccessTokenKeyIdsResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'client_assertion_type',
            'client_assertion'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method gets_all_valid_channel_access_token_key_ids" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('client_assertion_type') is not None:  # noqa: E501
            _query_params.append(('client_assertion_type', _params['client_assertion_type']))

        if _params.get('client_assertion') is not None:  # noqa: E501
            _query_params.append(('client_assertion', _params['client_assertion']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "ChannelAccessTokenKeyIdsResponse",
        }

        return self.api_client.call_api(
            '/oauth2/v2.1/tokens/kid', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def issue_channel_token(self, grant_type : Annotated[StrictStr, Field(..., description="`client_credentials`")], client_id : Annotated[StrictStr, Field(..., description="Channel ID.")], client_secret : Annotated[StrictStr, Field(..., description="Channel secret.")], **kwargs) -> IssueShortLivedChannelAccessTokenResponse:  # noqa: E501
        """issue_channel_token  # noqa: E501

        Issue short-lived channel access token  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.issue_channel_token(grant_type, client_id, client_secret, async_req=True)
        >>> result = thread.get()

        :param grant_type: `client_credentials` (required)
        :type grant_type: str
        :param client_id: Channel ID. (required)
        :type client_id: str
        :param client_secret: Channel secret. (required)
        :type client_secret: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: IssueShortLivedChannelAccessTokenResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the issue_channel_token_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.issue_channel_token_with_http_info(grant_type, client_id, client_secret, **kwargs)  # noqa: E501

    @validate_arguments
    def issue_channel_token_with_http_info(self, grant_type : Annotated[StrictStr, Field(..., description="`client_credentials`")], client_id : Annotated[StrictStr, Field(..., description="Channel ID.")], client_secret : Annotated[StrictStr, Field(..., description="Channel secret.")], **kwargs) -> ApiResponse:  # noqa: E501
        """issue_channel_token  # noqa: E501

        Issue short-lived channel access token  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.issue_channel_token_with_http_info(grant_type, client_id, client_secret, async_req=True)
        >>> result = thread.get()

        :param grant_type: `client_credentials` (required)
        :type grant_type: str
        :param client_id: Channel ID. (required)
        :type client_id: str
        :param client_secret: Channel secret. (required)
        :type client_secret: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(IssueShortLivedChannelAccessTokenResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'grant_type',
            'client_id',
            'client_secret'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method issue_channel_token" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        if _params['grant_type']:
            _form_params.append(('grant_type', _params['grant_type']))

        if _params['client_id']:
            _form_params.append(('client_id', _params['client_id']))

        if _params['client_secret']:
            _form_params.append(('client_secret', _params['client_secret']))

        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/x-www-form-urlencoded']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "IssueShortLivedChannelAccessTokenResponse",
            '400': "ErrorResponse",
        }

        return self.api_client.call_api(
            '/v2/oauth/accessToken', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def issue_channel_token_by_jwt(self, grant_type : Annotated[StrictStr, Field(..., description="client_credentials")], client_assertion_type : Annotated[StrictStr, Field(..., description="urn:ietf:params:oauth:client-assertion-type:jwt-bearer")], client_assertion : Annotated[StrictStr, Field(..., description="A JSON Web Token the client needs to create and sign with the private key of the Assertion Signing Key.")], **kwargs) -> IssueChannelAccessTokenResponse:  # noqa: E501
        """issue_channel_token_by_jwt  # noqa: E501

        Issues a channel access token that allows you to specify a desired expiration date. This method lets you use JWT assertion for authentication.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.issue_channel_token_by_jwt(grant_type, client_assertion_type, client_assertion, async_req=True)
        >>> result = thread.get()

        :param grant_type: client_credentials (required)
        :type grant_type: str
        :param client_assertion_type: urn:ietf:params:oauth:client-assertion-type:jwt-bearer (required)
        :type client_assertion_type: str
        :param client_assertion: A JSON Web Token the client needs to create and sign with the private key of the Assertion Signing Key. (required)
        :type client_assertion: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: IssueChannelAccessTokenResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the issue_channel_token_by_jwt_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.issue_channel_token_by_jwt_with_http_info(grant_type, client_assertion_type, client_assertion, **kwargs)  # noqa: E501

    @validate_arguments
    def issue_channel_token_by_jwt_with_http_info(self, grant_type : Annotated[StrictStr, Field(..., description="client_credentials")], client_assertion_type : Annotated[StrictStr, Field(..., description="urn:ietf:params:oauth:client-assertion-type:jwt-bearer")], client_assertion : Annotated[StrictStr, Field(..., description="A JSON Web Token the client needs to create and sign with the private key of the Assertion Signing Key.")], **kwargs) -> ApiResponse:  # noqa: E501
        """issue_channel_token_by_jwt  # noqa: E501

        Issues a channel access token that allows you to specify a desired expiration date. This method lets you use JWT assertion for authentication.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.issue_channel_token_by_jwt_with_http_info(grant_type, client_assertion_type, client_assertion, async_req=True)
        >>> result = thread.get()

        :param grant_type: client_credentials (required)
        :type grant_type: str
        :param client_assertion_type: urn:ietf:params:oauth:client-assertion-type:jwt-bearer (required)
        :type client_assertion_type: str
        :param client_assertion: A JSON Web Token the client needs to create and sign with the private key of the Assertion Signing Key. (required)
        :type client_assertion: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(IssueChannelAccessTokenResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'grant_type',
            'client_assertion_type',
            'client_assertion'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method issue_channel_token_by_jwt" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        if _params['grant_type']:
            _form_params.append(('grant_type', _params['grant_type']))

        if _params['client_assertion_type']:
            _form_params.append(('client_assertion_type', _params['client_assertion_type']))

        if _params['client_assertion']:
            _form_params.append(('client_assertion', _params['client_assertion']))

        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/x-www-form-urlencoded']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "IssueChannelAccessTokenResponse",
        }

        return self.api_client.call_api(
            '/oauth2/v2.1/token', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def issue_stateless_channel_token(self, grant_type : Annotated[StrictStr, Field(..., description="`client_credentials`")], client_assertion_type : Annotated[StrictStr, Field(..., description="URL-encoded value of `urn:ietf:params:oauth:client-assertion-type:jwt-bearer`")], client_assertion : Annotated[StrictStr, Field(..., description="A JSON Web Token the client needs to create and sign with the private key of the Assertion Signing Key.")], client_id : Annotated[StrictStr, Field(..., description="Channel ID.")], client_secret : Annotated[StrictStr, Field(..., description="Channel secret.")], **kwargs) -> IssueStatelessChannelAccessTokenResponse:  # noqa: E501
        """issue_stateless_channel_token  # noqa: E501

        Issues a new stateless channel access token, which doesn't have max active token limit unlike the other token types. The newly issued token is only valid for 15 minutes but can not be revoked until it naturally expires.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.issue_stateless_channel_token(grant_type, client_assertion_type, client_assertion, client_id, client_secret, async_req=True)
        >>> result = thread.get()

        :param grant_type: `client_credentials` (required)
        :type grant_type: str
        :param client_assertion_type: URL-encoded value of `urn:ietf:params:oauth:client-assertion-type:jwt-bearer` (required)
        :type client_assertion_type: str
        :param client_assertion: A JSON Web Token the client needs to create and sign with the private key of the Assertion Signing Key. (required)
        :type client_assertion: str
        :param client_id: Channel ID. (required)
        :type client_id: str
        :param client_secret: Channel secret. (required)
        :type client_secret: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: IssueStatelessChannelAccessTokenResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the issue_stateless_channel_token_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.issue_stateless_channel_token_with_http_info(grant_type, client_assertion_type, client_assertion, client_id, client_secret, **kwargs)  # noqa: E501

    @validate_arguments
    def issue_stateless_channel_token_with_http_info(self, grant_type : Annotated[StrictStr, Field(..., description="`client_credentials`")], client_assertion_type : Annotated[StrictStr, Field(..., description="URL-encoded value of `urn:ietf:params:oauth:client-assertion-type:jwt-bearer`")], client_assertion : Annotated[StrictStr, Field(..., description="A JSON Web Token the client needs to create and sign with the private key of the Assertion Signing Key.")], client_id : Annotated[StrictStr, Field(..., description="Channel ID.")], client_secret : Annotated[StrictStr, Field(..., description="Channel secret.")], **kwargs) -> ApiResponse:  # noqa: E501
        """issue_stateless_channel_token  # noqa: E501

        Issues a new stateless channel access token, which doesn't have max active token limit unlike the other token types. The newly issued token is only valid for 15 minutes but can not be revoked until it naturally expires.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.issue_stateless_channel_token_with_http_info(grant_type, client_assertion_type, client_assertion, client_id, client_secret, async_req=True)
        >>> result = thread.get()

        :param grant_type: `client_credentials` (required)
        :type grant_type: str
        :param client_assertion_type: URL-encoded value of `urn:ietf:params:oauth:client-assertion-type:jwt-bearer` (required)
        :type client_assertion_type: str
        :param client_assertion: A JSON Web Token the client needs to create and sign with the private key of the Assertion Signing Key. (required)
        :type client_assertion: str
        :param client_id: Channel ID. (required)
        :type client_id: str
        :param client_secret: Channel secret. (required)
        :type client_secret: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(IssueStatelessChannelAccessTokenResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'grant_type',
            'client_assertion_type',
            'client_assertion',
            'client_id',
            'client_secret'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method issue_stateless_channel_token" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        if _params['grant_type']:
            _form_params.append(('grant_type', _params['grant_type']))

        if _params['client_assertion_type']:
            _form_params.append(('client_assertion_type', _params['client_assertion_type']))

        if _params['client_assertion']:
            _form_params.append(('client_assertion', _params['client_assertion']))

        if _params['client_id']:
            _form_params.append(('client_id', _params['client_id']))

        if _params['client_secret']:
            _form_params.append(('client_secret', _params['client_secret']))

        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/x-www-form-urlencoded']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "IssueStatelessChannelAccessTokenResponse",
        }

        return self.api_client.call_api(
            '/oauth2/v3/token', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def revoke_channel_token(self, access_token : Annotated[StrictStr, Field(..., description="Channel access token")], **kwargs) -> None:  # noqa: E501
        """revoke_channel_token  # noqa: E501

        Revoke short-lived or long-lived channel access token  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.revoke_channel_token(access_token, async_req=True)
        >>> result = thread.get()

        :param access_token: Channel access token (required)
        :type access_token: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the revoke_channel_token_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.revoke_channel_token_with_http_info(access_token, **kwargs)  # noqa: E501

    @validate_arguments
    def revoke_channel_token_with_http_info(self, access_token : Annotated[StrictStr, Field(..., description="Channel access token")], **kwargs) -> ApiResponse:  # noqa: E501
        """revoke_channel_token  # noqa: E501

        Revoke short-lived or long-lived channel access token  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.revoke_channel_token_with_http_info(access_token, async_req=True)
        >>> result = thread.get()

        :param access_token: Channel access token (required)
        :type access_token: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'access_token'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method revoke_channel_token" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        if _params['access_token']:
            _form_params.append(('access_token', _params['access_token']))

        # process the body parameter
        _body_params = None
        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/x-www-form-urlencoded']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/v2/oauth/revoke', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def revoke_channel_token_by_jwt(self, client_id : Annotated[StrictStr, Field(..., description="Channel ID")], client_secret : Annotated[StrictStr, Field(..., description="Channel Secret")], access_token : Annotated[StrictStr, Field(..., description="Channel access token")], **kwargs) -> None:  # noqa: E501
        """revoke_channel_token_by_jwt  # noqa: E501

        Revoke channel access token v2.1  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.revoke_channel_token_by_jwt(client_id, client_secret, access_token, async_req=True)
        >>> result = thread.get()

        :param client_id: Channel ID (required)
        :type client_id: str
        :param client_secret: Channel Secret (required)
        :type client_secret: str
        :param access_token: Channel access token (required)
        :type access_token: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the revoke_channel_token_by_jwt_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.revoke_channel_token_by_jwt_with_http_info(client_id, client_secret, access_token, **kwargs)  # noqa: E501

    @validate_arguments
    def revoke_channel_token_by_jwt_with_http_info(self, client_id : Annotated[StrictStr, Field(..., description="Channel ID")], client_secret : Annotated[StrictStr, Field(..., description="Channel Secret")], access_token : Annotated[StrictStr, Field(..., description="Channel access token")], **kwargs) -> ApiResponse:  # noqa: E501
        """revoke_channel_token_by_jwt  # noqa: E501

        Revoke channel access token v2.1  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.revoke_channel_token_by_jwt_with_http_info(client_id, client_secret, access_token, async_req=True)
        >>> result = thread.get()

        :param client_id: Channel ID (required)
        :type client_id: str
        :param client_secret: Channel Secret (required)
        :type client_secret: str
        :param access_token: Channel access token (required)
        :type access_token: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'client_id',
            'client_secret',
            'access_token'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method revoke_channel_token_by_jwt" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        if _params['client_id']:
            _form_params.append(('client_id', _params['client_id']))

        if _params['client_secret']:
            _form_params.append(('client_secret', _params['client_secret']))

        if _params['access_token']:
            _form_params.append(('access_token', _params['access_token']))

        # process the body parameter
        _body_params = None
        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/x-www-form-urlencoded']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/oauth2/v2.1/revoke', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def verify_channel_token(self, access_token : Annotated[StrictStr, Field(..., description="A short-lived or long-lived channel access token.")], **kwargs) -> VerifyChannelAccessTokenResponse:  # noqa: E501
        """verify_channel_token  # noqa: E501

        Verify the validity of short-lived and long-lived channel access tokens  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.verify_channel_token(access_token, async_req=True)
        >>> result = thread.get()

        :param access_token: A short-lived or long-lived channel access token. (required)
        :type access_token: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: VerifyChannelAccessTokenResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the verify_channel_token_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.verify_channel_token_with_http_info(access_token, **kwargs)  # noqa: E501

    @validate_arguments
    def verify_channel_token_with_http_info(self, access_token : Annotated[StrictStr, Field(..., description="A short-lived or long-lived channel access token.")], **kwargs) -> ApiResponse:  # noqa: E501
        """verify_channel_token  # noqa: E501

        Verify the validity of short-lived and long-lived channel access tokens  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.verify_channel_token_with_http_info(access_token, async_req=True)
        >>> result = thread.get()

        :param access_token: A short-lived or long-lived channel access token. (required)
        :type access_token: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(VerifyChannelAccessTokenResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'access_token'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method verify_channel_token" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        if _params['access_token']:
            _form_params.append(('access_token', _params['access_token']))

        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/x-www-form-urlencoded']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "VerifyChannelAccessTokenResponse",
        }

        return self.api_client.call_api(
            '/v2/oauth/verify', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def verify_channel_token_by_jwt(self, access_token : Annotated[StrictStr, Field(..., description="Channel access token with a user-specified expiration (Channel Access Token v2.1).")], **kwargs) -> VerifyChannelAccessTokenResponse:  # noqa: E501
        """verify_channel_token_by_jwt  # noqa: E501

        You can verify whether a Channel access token with a user-specified expiration (Channel Access Token v2.1) is valid.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.verify_channel_token_by_jwt(access_token, async_req=True)
        >>> result = thread.get()

        :param access_token: Channel access token with a user-specified expiration (Channel Access Token v2.1). (required)
        :type access_token: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: VerifyChannelAccessTokenResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the verify_channel_token_by_jwt_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.verify_channel_token_by_jwt_with_http_info(access_token, **kwargs)  # noqa: E501

    @validate_arguments
    def verify_channel_token_by_jwt_with_http_info(self, access_token : Annotated[StrictStr, Field(..., description="Channel access token with a user-specified expiration (Channel Access Token v2.1).")], **kwargs) -> ApiResponse:  # noqa: E501
        """verify_channel_token_by_jwt  # noqa: E501

        You can verify whether a Channel access token with a user-specified expiration (Channel Access Token v2.1) is valid.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.verify_channel_token_by_jwt_with_http_info(access_token, async_req=True)
        >>> result = thread.get()

        :param access_token: Channel access token with a user-specified expiration (Channel Access Token v2.1). (required)
        :type access_token: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(VerifyChannelAccessTokenResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'access_token'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method verify_channel_token_by_jwt" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('access_token') is not None:  # noqa: E501
            _query_params.append(('access_token', _params['access_token']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "VerifyChannelAccessTokenResponse",
        }

        return self.api_client.call_api(
            '/oauth2/v2.1/verify', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import sys
import unittest
from numbers import Number

from linebot.models import (
    Base,
)
from linebot.utils import to_camel_case

PY3 = sys.version_info[0] == 3


class SerializeTestCase(unittest.TestCase):
    MESSAGE = 'message'
    STICKER = 'sticker'
    POSTBACK = 'postback'
    CAMERA = 'camera'
    CAMERA_ROLL = 'cameraRoll'
    DATETIMEPICKER = 'datetimepicker'
    URI = 'uri'
    LOCATION = 'location'
    FLEX = 'flex'
    GENDER = "gender"
    APP_TYPE = "appType"
    AGE = "age"
    AREA = "area"
    SUBSCRIPTION_PERIOD = "subscriptionPeriod"
    SPAN = 'span'
    BUBBLE = 'bubble'
    CAROUSEL = 'carousel'
    BOX = 'box'
    BUTTON = 'button'
    FILLER = 'filler'
    ICON = 'icon'
    TEXT = 'text'
    IMAGE = 'image'
    VIDEO = 'video'
    AUDIO = 'audio'
    SEPARATOR = 'separator'
    IMAGEMAP = 'imagemap'
    ACTION = 'action'
    TEMPLATE = 'template'
    BUTTONS = 'buttons'
    CONFIRM = 'confirm'
    IMAGE_CAROUSEL = 'image_carousel'
    LINEAR_GRADIENT = 'linearGradient'

    def serialize_as_dict(self, obj, type=None):
        if isinstance(obj, Base):
            return obj.as_json_dict()
        elif isinstance(obj, dict):
            ret = {to_camel_case(k): self.serialize_as_dict(v) for k, v in obj.items()}
            if type is not None:
                ret['type'] = type
            return ret
        elif isinstance(obj, list):
            return [self.serialize_as_dict(elem) for elem in obj]
        else:
            if PY3:
                self.assertIsInstance(obj, (str, bool, Number))
            else:
                self.assertIsInstance(obj, (basestring, bool, Number))  # noqa
            return obj

    class ConstError(TypeError):
        pass

    def __setattr__(self, name, value):
        if name in SerializeTestCase.__dict__:
            raise self.ConstError("Can't rebind const (%s)" % name)
        self.__dict__[name] = value



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.things module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .base import Base

from deprecated import deprecated

from linebot.deprecations import (
    LineBotSdkDeprecatedIn30
)


@deprecated(reason="Use 'from linebot.v3.webhooks import ThingsContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class Things(with_metaclass(ABCMeta, Base)):
    """Abstract Base Class of Things."""

    def __init__(self, device_id=None, **kwargs):
        """__init__ method.

        :param str device_id: Device ID.
        :param kwargs:
        """
        super(Things, self).__init__(**kwargs)
        self.device_id = device_id


@deprecated(reason="Use 'from linebot.v3.webhooks import LinkThingsContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class DeviceLink(Things):
    """DeviceLink.

    https://developers.line.biz/en/reference/messaging-api/#device-link-event

    Indicates that a user linked a device with LINE.
    """

    def __init__(self, device_id=None, **kwargs):
        """__init__ method.

        :param str device_id: Device ID of the device that has been linked with LINE.
        :param kwargs:
        """
        super(DeviceLink, self).__init__(device_id=device_id, **kwargs)

        self.type = 'link'


@deprecated(reason="Use 'from linebot.v3.webhooks import UnlinkThingsContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class DeviceUnlink(Things):
    """DeviceUnlink.

    https://developers.line.biz/en/reference/messaging-api/#device-unlink-event

    Indicates that the user unlinked a device from LINE.
    """

    def __init__(self, device_id=None, **kwargs):
        """__init__ method.

        :param str device_id: Device ID of the device that was unlinked from LINE.
        :param kwargs:
        """
        super(DeviceUnlink, self).__init__(device_id=device_id, **kwargs)

        self.type = 'unlink'


@deprecated(reason="Use 'from linebot.v3.webhooks import ScenarioResultThingsContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class ScenarioResult(Things):
    """ScenarioResult.

    https://developers.line.biz/en/reference/messaging-api/#scenario-result-event

    Indicates that an automatic communication scenario has been executed.
    """

    def __init__(self, device_id=None, result=None, **kwargs):
        """__init__ method.

        :param str device_id: Device ID of the device that executed the scenario.
        :param str result: ScenarioResultPayload object.
        :param kwargs:
        """
        super(ScenarioResult, self).__init__(device_id=device_id, **kwargs)

        self.type = 'scenarioResult'
        self.result = self.get_or_new_from_json_dict(
            result, ScenarioResultPayload
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import ScenarioResult' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class ScenarioResultPayload(Base):
    """ScenarioResultPayload."""

    def __init__(self, scenario_id=None, revision=None, start_time=None,
                 result_code=None, end_time=None, action_results=None,
                 ble_notification_payload=None, error_reason=None, **kwargs):
        """__init__ method.

        :param str scenario_id: Scenario ID executed.
        :param long revision: Revision number.
        :param long start_time: Timestamp for when execution of scenario
            action started (milliseconds).
        :param long end_time: Timestamp for when execution of scenario
            was completed (milliseconds).
        :param str result_code: Scenario execution completion status.
        :param action_results: Array of actions specified in a scenario.
        :type action_results: list[T <= :py:class:`linebot.models.things.ActionResult`]
        :param str ble_notification_payload: Data contained in notification.
        :param str error_reason: Error response.
        :param kwargs:
        """
        super(ScenarioResultPayload, self).__init__(**kwargs)

        self.scenario_id = scenario_id
        self.revision = revision
        self.start_time = start_time
        self.end_time = end_time
        self.result_code = result_code
        self.action_results = [self.get_or_new_from_json_dict(it, ActionResult)
                               for it in action_results]
        self.ble_notification_payload = ble_notification_payload
        self.error_reason = error_reason


@deprecated(reason="Use 'from linebot.v3.webhooks import ActionResult' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class ActionResult(Base):
    """ActionResult.

    Execution result of individual operations specified in action
    """

    def __init__(self, type=None, data=None, **kwargs):
        """__init__ method.

        :param str type: Type of the executed action.
        :param str data: Base64-encoded binary data.
        :param kwargs:
        """
        super(ActionResult, self).__init__(**kwargs)

        self.type = type
        self.data = data



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.rich_menu module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .actions import get_action
from .base import Base


class RichMenu(with_metaclass(ABCMeta, Base)):
    """RichMenu.

    https://developers.line.me/en/docs/messaging-api/reference/#rich-menu-object
    """

    def __init__(self, size=None, selected=None, name=None, chat_bar_text=None,
                 areas=None, **kwargs):
        """__init__ method.

        :param size: size object which describe the rich menu displayed in the chat.
            Rich menu images must be one of the following sizes: 2500x1686, 2500x843.
        :type size: :py:class:`linebot.models.rich_menu.RichMenuSize`
        :param bool selected: true to display the rich menu by default. Otherwise, false.
        :param str name: Name of the rich menu.
            Maximum of 300 characters.
        :param str chatBarText: Text displayed in the chat bar.
                                Maximum of 14 characters.
        :param areas: Array of area objects which define coordinates and size of tappable areas.
                      Maximum of 20 area objects.
        :type areas: list[T <= :py:class:`linebot.models.rich_menu.RichMenuArea`]
        :param kwargs:
        """
        super(RichMenu, self).__init__(**kwargs)

        self.size = self.get_or_new_from_json_dict(size, RichMenuSize)
        self.selected = selected
        self.name = name
        self.chat_bar_text = chat_bar_text

        new_areas = []
        if areas:
            for area in areas:
                new_areas.append(
                    self.get_or_new_from_json_dict(area, RichMenuArea)
                )
        self.areas = new_areas


class RichMenuSize(with_metaclass(ABCMeta, Base)):
    """RichMenuSize.

    https://developers.line.me/en/docs/messaging-api/reference/#size-object
    """

    def __init__(self, width=None, height=None, **kwargs):
        """__init__ method.

        :param int width: Width of the rich menu. Must be 2500.
        :param int height: Height of the rich menu. Possible values: 1686, 843.
        :param kwargs:
        """
        super(RichMenuSize, self).__init__(**kwargs)

        self.width = width
        self.height = height


class RichMenuArea(with_metaclass(ABCMeta, Base)):
    """RichMenuArea.

    https://developers.line.me/en/docs/messaging-api/reference/#area-object
    """

    def __init__(self, bounds=None, action=None, **kwargs):
        """__init__ method.

        :param bounds: Object describing the boundaries of the area in pixels. See bounds object.
        :type bounds: :py:class:`linebot.models.rich_menu.RichMenuBound`
        :param action: Action performed when the area is tapped. See action objects.
        :type action: T <= :py:class:`linebot.models.actions.Action`
        :param kwargs:
        """
        super(RichMenuArea, self).__init__(**kwargs)

        self.bounds = self.get_or_new_from_json_dict(bounds, RichMenuBounds)
        self.action = get_action(action)


class RichMenuBounds(with_metaclass(ABCMeta, Base)):
    """RichMenuBounds.

    https://developers.line.me/en/docs/messaging-api/reference/#bounds-object
    """

    def __init__(self, x=None, y=None, width=None, height=None, **kwargs):
        """__init__ method.

        :param int x: Horizontal position relative to the top-left corner of the area.
        :param int y: Vertical position relative to the top-left corner of the area.
        :param int width: Width of the area.
        :param int height: Height of the area.
        :param kwargs:
        """
        super(RichMenuBounds, self).__init__(**kwargs)

        self.x = x
        self.y = y
        self.width = width
        self.height = height


class RichMenuAlias(with_metaclass(ABCMeta, Base)):
    """RichMenuAlias.

    https://developers.line.biz/en/reference/messaging-api/#create-rich-menu-alias
    """

    def __init__(self, rich_menu_alias_id=None, rich_menu_id=None, **kwargs):
        """__init__ method.

        :param string rich_menu_alias_id: Rich menu alias ID,
         which can be any ID, unique for each channel.
        :param string rich_menu_id: The rich menu ID to be associated with the rich menu alias.
        :param kwargs:
        """
        super(RichMenuAlias, self).__init__(**kwargs)

        self.rich_menu_alias_id = rich_menu_alias_id
        self.rich_menu_id = rich_menu_id



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.events module."""


from abc import ABCMeta

from future.utils import with_metaclass

from linebot.models.base import Base
from linebot.models.delivery_context import DeliveryContext
from linebot.models.messages import (
    TextMessage,
    ImageMessage,
    VideoMessage,
    AudioMessage,
    LocationMessage,
    StickerMessage,
    FileMessage
)
from linebot.models.sources import SourceUser, SourceGroup, SourceRoom
from linebot.models.things import (
    DeviceUnlink,
    DeviceLink,
    ScenarioResult,
)
from linebot.models.things import Things  # noqa, backward compatibility
from linebot.models.unsend import Unsend
from linebot.models.video_play_complete import VideoPlayComplete

from deprecated import deprecated

from deprecated import deprecated

from linebot.deprecations import (
    LineBotSdkDeprecatedIn30
)


@deprecated(reason="Use 'from linebot.v3.webhooks import VideoPlayComplete' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class Event(with_metaclass(ABCMeta, Base)):
    """Abstract Base Class of Webhook Event.

    https://developers.line.biz/en/reference/messaging-api/#webhook-event-objects
    """

    def __init__(
        self,
        mode=None,
        timestamp=None,
        source=None,
        webhook_event_id=None,
        delivery_context=None,
        **kwargs
    ):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param kwargs:
        """
        super(Event, self).__init__(**kwargs)

        self.type = None
        self.mode = mode
        self.timestamp = timestamp
        self.source = self.get_or_new_from_json_dict_with_types(
            source, {
                'user': SourceUser,
                'group': SourceGroup,
                'room': SourceRoom,
            }
        )
        self.webhook_event_id = webhook_event_id
        self.delivery_context = self.get_or_new_from_json_dict(
            delivery_context, DeliveryContext
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import MessageEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class MessageEvent(Event):
    """Webhook MessageEvent.

    https://developers.line.biz/en/reference/messaging-api/#message-event

    Event object which contains the sent message.
    The message field contains a message object which corresponds with the message type.
    You can reply to message events.
    """

    def __init__(self, mode=None, timestamp=None, source=None, reply_token=None, message=None,
                 use_raw_message=False, **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param str reply_token: Reply token
        :param message: Message object
        :type message: T <= :py:class:`linebot.models.messages.Message`
        :param bool use_raw_message: Using original Message key as attribute
        :param kwargs:
        """
        super(MessageEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'message'
        self.reply_token = reply_token
        self.message = self.get_or_new_from_json_dict_with_types(
            message, {
                'text': TextMessage,
                'image': ImageMessage,
                'video': VideoMessage,
                'audio': AudioMessage,
                'location': LocationMessage,
                'sticker': StickerMessage,
                'file': FileMessage
            }, use_raw_message=use_raw_message
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import FollowEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class FollowEvent(Event):
    """Webhook FollowEvent.

    https://developers.line.biz/en/reference/messaging-api/#follow-event

    Event object for when your account is added as a friend (or unblocked).
    You can reply to follow events.
    """

    def __init__(self, mode=None, timestamp=None, source=None, reply_token=None, **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param str reply_token: Reply token
        :param kwargs:
        """
        super(FollowEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'follow'
        self.reply_token = reply_token


@deprecated(reason="Use 'from linebot.v3.webhooks import UnfollowEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class UnfollowEvent(Event):
    """Webhook UnfollowEvent.

    https://developers.line.biz/en/reference/messaging-api/#unfollow-event

    Event object for when your account is blocked.
    """

    def __init__(self, mode=None, timestamp=None, source=None, **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param kwargs:
        """
        super(UnfollowEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'unfollow'


@deprecated(reason="Use 'from linebot.v3.webhooks import JoinEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class JoinEvent(Event):
    """Webhook JoinEvent.

    https://developers.line.biz/en/reference/messaging-api/#join-event

    Event object for when your account joins a group or talk room.
    You can reply to join events.
    """

    def __init__(self, mode=None, timestamp=None, source=None, reply_token=None, **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param str reply_token: Reply token
        :param kwargs:
        """
        super(JoinEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'join'
        self.reply_token = reply_token


@deprecated(reason="Use 'from linebot.v3.webhooks import LeaveEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class LeaveEvent(Event):
    """Webhook LeaveEvent.

    https://developers.line.biz/en/reference/messaging-api/#leave-event

    Event object for when your account leaves a group.
    """

    def __init__(self, mode=None, timestamp=None, source=None, **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param kwargs:
        """
        super(LeaveEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'leave'


@deprecated(reason="Use 'from linebot.v3.webhooks import PostbackEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class PostbackEvent(Event):
    """Webhook PostbackEvent.

    https://developers.line.biz/en/reference/messaging-api/#postback-event

    Event object for when a user performs an action on
    a template message which initiates a postback.
    You can reply to postback events.
    """

    def __init__(self, mode=None, timestamp=None, source=None, reply_token=None, postback=None,
                 **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param str reply_token: Reply token
        :param postback: Postback object
        :type postback: :py:class:`linebot.models.events.Postback`
        :param kwargs:
        """
        super(PostbackEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'postback'
        self.reply_token = reply_token
        self.postback = self.get_or_new_from_json_dict(
            postback, Postback
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import BeaconEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class BeaconEvent(Event):
    """Webhook BeaconEvent.

    https://developers.line.biz/en/reference/messaging-api/#beacon-event

    Event object for when a user detects a LINE Beacon. You can reply to beacon events.
    """

    def __init__(self, mode=None, timestamp=None, source=None, reply_token=None, beacon=None,
                 **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param str reply_token: Reply token
        :param beacon: Beacon object
        :type beacon: :py:class:`linebot.models.events.Beacon`
        :param kwargs:
        """
        super(BeaconEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'beacon'
        self.reply_token = reply_token
        self.beacon = self.get_or_new_from_json_dict(
            beacon, Beacon
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import MemberJoinedEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class MemberJoinedEvent(Event):
    """Webhook MemberJoinedEvent.

    https://developers.line.biz/en/reference/messaging-api/#member-joined-event

    Event object for when a user joins a group or room that the bot is in.

    """

    def __init__(self, mode=None, timestamp=None, source=None, reply_token=None, joined=None,
                 **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param str reply_token: Reply token
        :param joined: Joined object
        :type joined: :py:class:`linebot.models.events.Joined`
        :param kwargs:
        """
        super(MemberJoinedEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'memberJoined'
        self.reply_token = reply_token
        self.joined = self.get_or_new_from_json_dict(
            joined, Joined
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import MemberLeftEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class MemberLeftEvent(Event):
    """Webhook MemberLeftEvent.

    https://developers.line.biz/en/reference/messaging-api/#member-left-event

    Event object for when a user leaves a group or room that the bot is in.

    """

    def __init__(self, mode=None, timestamp=None, source=None, left=None, **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param left: Left object
        :type left: :py:class:`linebot.models.events.Left`
        :param kwargs:
        """
        super(MemberLeftEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'memberLeft'
        self.left = self.get_or_new_from_json_dict(
            left, Left
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import AccountLinkEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class AccountLinkEvent(Event):
    """Webhook AccountLinkEvent.

    https://developers.line.me/en/docs/messaging-api/reference/#account-link-event

    Event object for when a user has linked his/her LINE account with a provider's service account.
    You can reply to account link events.
    If the link token has expired or has already been used,
    no webhook event will be sent and the user will be shown an error.
    """

    def __init__(self, mode=None, timestamp=None, source=None, reply_token=None, link=None,
                 **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param str reply_token: Reply token
        :param link: Link object
        :type link: :py:class:`linebot.models.events.Link`
        :param kwargs:
        """
        super(AccountLinkEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'accountLink'
        self.reply_token = reply_token
        self.link = self.get_or_new_from_json_dict(
            link, Link
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import ThingsEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class ThingsEvent(Event):
    """Webhook ThingsEvent.

    https://developers.line.biz/en/reference/messaging-api/#device-link-event
    https://developers.line.biz/en/reference/messaging-api/#device-unlink-event
    https://developers.line.biz/en/reference/messaging-api/#scenario-result-event

    Event sent from LINE Things Webhook service.
    """

    def __init__(self, mode=None, timestamp=None, source=None, reply_token=None, things=None,
                 **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param str reply_token: Reply token
        :param things: Things object
        :type things: T <= :py:class:`linebot.models.things.Things`
        :param kwargs:
        """
        super(ThingsEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'things'
        self.reply_token = reply_token
        self.things = self.get_or_new_from_json_dict_with_types(
            things, {
                'link': DeviceLink,
                'unlink': DeviceUnlink,
                'scenarioResult': ScenarioResult,
            }
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import UnsendEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class UnsendEvent(Event):
    """Webhook UnsendEvent.

    https://developers.line.biz/en/reference/messaging-api/#unsend-event

    Event object for when the user unsends a message in a group or room.
    """

    def __init__(self, mode=None, timestamp=None, source=None, unsend=None, **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param unsend: Unsend object
        :type unsend: T <= :py:class:`linebot.models.unsend.Unsend`
        :param kwargs:
        """
        super(UnsendEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'unsend'
        self.unsend = self.get_or_new_from_json_dict(
            unsend, Unsend
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import VideoPlayCompleteEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class VideoPlayCompleteEvent(Event):
    """Webhook VideoCompleteEvent.

    https://developers.line.biz/en/reference/messaging-api/#video-viewing-complete

    Event object Event for when a user finishes viewing a video at least once.
    """

    def __init__(self, mode=None, timestamp=None, source=None, reply_token=None,
                 video_play_complete=None, **kwargs):
        """__init__ method.

        :param str mode: Channel state
        :param long timestamp: Time of the event in milliseconds
        :param source: Source object
        :type source: T <= :py:class:`linebot.models.sources.Source`
        :param str reply_token: Reply token
        :param video_play_complete: VideoPlayComplete object
        :type video_play_complete:
            T <= :py:class:`linebot.models.video_play_complete.VideoPlayComplete`
        :param kwargs:
        """
        super(VideoPlayCompleteEvent, self).__init__(
            mode=mode, timestamp=timestamp, source=source, **kwargs
        )

        self.type = 'videoPlayComplete'
        self.reply_token = reply_token
        self.video_play_complete = self.get_or_new_from_json_dict(
            video_play_complete, VideoPlayComplete
        )


@deprecated(reason="Use 'from linebot.v3.models import UnknownEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class UnknownEvent(Event):
    """Unknown event.

    We welcome your contribution to line-bot-sdk-python!
    """

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(UnknownEvent, self).__init__(**kwargs)

        self.type = 'unknown'


@deprecated(reason="Use 'from linebot.v3.webhooks import PostbackContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class Postback(Base):
    """Postback.

    https://developers.line.biz/en/reference/messaging-api/#postback-event
    """

    def __init__(self, data=None, params=None, **kwargs):
        """__init__ method.

        :param str data: Postback data
        :param dict params: JSON object with the date and time
            selected by a user through a datetime picker action.
            Only returned for postback actions via the datetime picker.
        :param kwargs:
        """
        super(Postback, self).__init__(**kwargs)

        self.data = data
        self.params = params


@deprecated(reason="Use 'from linebot.v3.webhooks import BeaconContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class Beacon(Base):
    """Beacon.

    https://developers.line.biz/en/reference/messaging-api/#beacon-event
    """

    def __init__(self, type=None, hwid=None, dm=None, **kwargs):
        """__init__ method.

        :param str type: Type of beacon event
        :param str hwid: Hardware ID of the beacon that was detected
        :param str dm: Optional. Device message of beacon which is hex string
        :param kwargs:
        """
        super(Beacon, self).__init__(**kwargs)

        self.type = type
        self.hwid = hwid
        self.dm = dm

    @property
    def device_message(self):
        """Get dm(device_message) as bytearray.

        :rtype: bytearray
        """
        return bytearray.fromhex(self.dm) if self.dm is not None else None


@deprecated(reason="Use 'from linebot.v3.webhooks import JoinedMembers' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class Joined(Base):
    """Joined.

    https://developers.line.biz/en/reference/messaging-api/#member-joined-event
    """

    def __init__(self, members=None, **kwargs):
        """__init__ method.

        :param dict members: Member of users who joined
        :param kwargs:
        """
        super(Joined, self).__init__(**kwargs)

        self._members = members

    @property
    def members(self):
        """Get members as list of SourceUser."""
        return [SourceUser(user_id=x['userId']) for x in self._members]


@deprecated(reason="Use 'from linebot.v3.webhooks import LeftMembers' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class Left(Base):
    """Left.

    https://developers.line.biz/en/reference/messaging-api/#member-left-event
    """

    def __init__(self, members=None, **kwargs):
        """__init__ method.

        :param dict members: Member of users who joined
        :param kwargs:
        """
        super(Left, self).__init__(**kwargs)

        self._members = members

    @property
    def members(self):
        """Get members as list of SourceUser."""
        return [SourceUser(user_id=x['userId']) for x in self._members]


@deprecated(reason="Use 'from linebot.v3.webhooks import AccountLinkEvent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class Link(Base):
    """Link.

    https://developers.line.me/en/docs/messaging-api/reference/#link-object
    """

    def __init__(self, result=None, nonce=None, **kwargs):
        """__init__ method.

        :param str result: Indicate whether the link was successful or not.
        :param str nonce: Specified nonce when verifying the user ID.
        """
        super(Link, self).__init__(**kwargs)

        self.result = result
        self.nonce = nonce



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List
from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr, conlist, constr
from linebot.v3.messaging.models.rich_menu_area import RichMenuArea
from linebot.v3.messaging.models.rich_menu_size import RichMenuSize

class RichMenuResponse(BaseModel):
    """
    RichMenuResponse
    """
    rich_menu_id: StrictStr = Field(..., alias="richMenuId", description="ID of a rich menu")
    size: RichMenuSize = Field(...)
    selected: StrictBool = Field(..., description="`true` to display the rich menu by default. Otherwise, `false`.")
    name: constr(strict=True, max_length=300) = Field(..., description="Name of the rich menu. This value can be used to help manage your rich menus and is not displayed to users.")
    chat_bar_text: constr(strict=True, max_length=14) = Field(..., alias="chatBarText", description="Text displayed in the chat bar")
    areas: conlist(RichMenuArea, max_items=20) = Field(..., description="Array of area objects which define the coordinates and size of tappable areas")

    __properties = ["richMenuId", "size", "selected", "name", "chatBarText", "areas"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RichMenuResponse:
        """Create an instance of RichMenuResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of size
        if self.size:
            _dict['size'] = self.size.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in areas (list)
        _items = []
        if self.areas:
            for _item in self.areas:
                if _item:
                    _items.append(_item.to_dict())
            _dict['areas'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RichMenuResponse:
        """Create an instance of RichMenuResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RichMenuResponse.parse_obj(obj)

        _obj = RichMenuResponse.parse_obj({
            "rich_menu_id": obj.get("richMenuId"),
            "size": RichMenuSize.from_dict(obj.get("size")) if obj.get("size") is not None else None,
            "selected": obj.get("selected"),
            "name": obj.get("name"),
            "chat_bar_text": obj.get("chatBarText"),
            "areas": [RichMenuArea.from_dict(_item) for _item in obj.get("areas")] if obj.get("areas") is not None else None
        })
        return _obj




import json
import unittest
from urllib.parse import urlparse, parse_qs
from pytest_httpserver import HTTPServer
from linebot.v3.messaging import (
    Configuration,
    ApiClient,
    MessagingApi,
    CouponCreateRequest,
    LotteryAcquisitionConditionRequest,
    CouponDiscountRewardRequest,
    DiscountFixedPriceInfoRequest,
    CouponMessage
)


class TestCouponAPI(unittest.TestCase):

    def test_coupon_create(self):
        expected_coupon_id = "COUPON123"
        expected_response_create = {
            "couponId": expected_coupon_id
        }
        expected_body = {
            "acquisitionCondition": {
                "type": "lottery",
                "lotteryProbability": 50,
                "maxAcquireCount": 1000
            },
            "barcodeImageUrl": "https://example.com/barcode.png",
            "couponCode": "UNIQUECODE123",
            "description": "Get 100 Yen off your purchase",
            "endTimestamp": 1700000000,
            "imageUrl": "https://example.com/image.png",
            "maxUseCountPerTicket": 1,
            "startTimestamp": 1600000000,
            "title": "100 Yen OFF",
            "usageCondition": "Minimum purchase of 500 Yen",
            "reward": {
                "type": "discount",
                "priceInfo": {
                    "type": "fixed",
                    "fixedAmount": 100
                }
            },
            "visibility": "PUBLIC",
            "timezone": "ASIA_TOKYO"
        }

        with HTTPServer() as httpserver:
            httpserver.expect_request(
                uri="/v2/bot/coupon",
                method="POST",
            ).respond_with_json(
                expected_response_create,
                status=200
            )

            configuration = Configuration(
                access_token="dummy-channel-access-token",
                host=httpserver.url_for("/")
            )

            with ApiClient(configuration) as api_client:
                line_bot_api = MessagingApi(api_client)

                req = CouponCreateRequest(
                    acquisitionCondition=LotteryAcquisitionConditionRequest(
                        lotteryProbability=50,
                        maxAcquireCount=1000
                    ),
                    barcodeImageUrl="https://example.com/barcode.png",
                    couponCode="UNIQUECODE123",
                    description="Get 100 Yen off your purchase",
                    endTimestamp=1700000000,
                    imageUrl="https://example.com/image.png",
                    maxUseCountPerTicket=1,
                    startTimestamp=1600000000,
                    title="100 Yen OFF",
                    usageCondition="Minimum purchase of 500 Yen",
                    reward=CouponDiscountRewardRequest(
                        priceInfo=DiscountFixedPriceInfoRequest(
                            fixedAmount=100
                        )
                    ),
                    visibility="PUBLIC",
                    timezone="ASIA_TOKYO"
                )

                response = line_bot_api.create_coupon(coupon_create_request=req)

            self.assertEqual(response.coupon_id, expected_coupon_id)
            self.assertEqual(len(httpserver.log), 1)

            request, _ = httpserver.log[0]
            got_body = json.loads(request.data.decode('utf-8'))
            self.assertEqual(got_body, expected_body)

    def test_coupon_close(self):
        expected_coupon_id = "COUPON123"

        with HTTPServer() as httpserver:
            httpserver.expect_request(
                uri=f"/v2/bot/coupon/{expected_coupon_id}/close",
                method="PUT",
            ).respond_with_json({}, status=200)

            configuration = Configuration(
                access_token="dummy-channel-access-token",
                host=httpserver.url_for("/")
            )

            with ApiClient(configuration) as api_client:
                line_bot_api = MessagingApi(api_client)
                response = line_bot_api.close_coupon(expected_coupon_id)

            self.assertIsNone(response)
            self.assertEqual(len(httpserver.log), 1)

    def test_list_coupon(self):
        expected_response_list = {
            "items": [
                {"couponId": "COUPON123", "title": "Discount Coupon"},
                {"couponId": "COUPON456", "title": "Special Offer"}
            ],
            "next": "nextPageToken"
        }

        with HTTPServer() as httpserver:
            httpserver.expect_request(
                uri="/v2/bot/coupon",
                method="GET",
            ).respond_with_json(
                expected_response_list,
                status=200
            )

            configuration = Configuration(
                access_token="dummy-channel-access-token",
                host=httpserver.url_for("/")
            )

            with ApiClient(configuration) as api_client:
                line_bot_api = MessagingApi(api_client)
                response = line_bot_api.list_coupon(
                    status=["RUNNING", "CLOSED"],
                    limit=10,
                )

            self.assertEqual(len(response.items), 2)
            self.assertEqual(response.items[0].coupon_id, "COUPON123")
            self.assertEqual(response.items[0].title, "Discount Coupon")
            self.assertEqual(response.items[1].coupon_id, "COUPON456")
            self.assertEqual(response.items[1].title, "Special Offer")
            self.assertEqual(response.next, expected_response_list["next"])
            self.assertEqual(len(httpserver.log), 1)

            request, _ = httpserver.log[0]
            parsed_url = urlparse(request.url)
            query_params = parse_qs(parsed_url.query)

            self.assertIn("status", query_params)
            self.assertIn("limit", query_params)
            self.assertEqual(query_params["status"], ["RUNNING", "CLOSED"])
            self.assertEqual(query_params["limit"], ["10"])

    def test_get_coupon_detail(self):
        expected_coupon_id = "COUPON123"
        expected_response_detail = {
            "couponId": expected_coupon_id,
            "title": "Discount Coupon",
            "status": "RUNNING"
        }

        with HTTPServer() as httpserver:
            httpserver.expect_request(
                uri=f"/v2/bot/coupon/{expected_coupon_id}",
                method="GET",
            ).respond_with_json(
                expected_response_detail,
                status=200
            )

            configuration = Configuration(
                access_token="dummy-channel-access-token",
                host=httpserver.url_for("/")
            )

            with ApiClient(configuration) as api_client:
                line_bot_api = MessagingApi(api_client)
                response = line_bot_api.get_coupon_detail(expected_coupon_id)

            self.assertEqual(response.coupon_id, expected_coupon_id)
            self.assertEqual(response.title, expected_response_detail["title"])
            self.assertEqual(len(httpserver.log), 1)

    def test_coupon_message(self):
        expected_coupon_id = "COUPON123"
        coupon_message = CouponMessage(couponId=expected_coupon_id)
        self.assertEqual(coupon_message.to_dict()["couponId"], expected_coupon_id)
        self.assertEqual(coupon_message.to_dict()["type"], "coupon")


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel
from linebot.v3.messaging.models.flex_block_style import FlexBlockStyle

class FlexBubbleStyles(BaseModel):
    """
    FlexBubbleStyles
    """
    header: Optional[FlexBlockStyle] = None
    hero: Optional[FlexBlockStyle] = None
    body: Optional[FlexBlockStyle] = None
    footer: Optional[FlexBlockStyle] = None

    __properties = ["header", "hero", "body", "footer"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FlexBubbleStyles:
        """Create an instance of FlexBubbleStyles from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of header
        if self.header:
            _dict['header'] = self.header.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of hero
        if self.hero:
            _dict['hero'] = self.hero.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of body
        if self.body:
            _dict['body'] = self.body.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of footer
        if self.footer:
            _dict['footer'] = self.footer.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> FlexBubbleStyles:
        """Create an instance of FlexBubbleStyles from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return FlexBubbleStyles.parse_obj(obj)

        _obj = FlexBubbleStyles.parse_obj({
            "header": FlexBlockStyle.from_dict(obj.get("header")) if obj.get("header") is not None else None,
            "hero": FlexBlockStyle.from_dict(obj.get("hero")) if obj.get("hero") is not None else None,
            "body": FlexBlockStyle.from_dict(obj.get("body")) if obj.get("body") is not None else None,
            "footer": FlexBlockStyle.from_dict(obj.get("footer")) if obj.get("footer") is not None else None
        })
        return _obj




# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json
import linebot.v3.webhooks.models


from typing import Optional, Union
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr
from linebot.v3.webhooks.models.delivery_context import DeliveryContext
from linebot.v3.webhooks.models.event_mode import EventMode
from linebot.v3.webhooks.models.source import Source

class Event(BaseModel):
    """
    Webhook event
    """
    type: StrictStr = Field(..., description="Type of the event")
    source: Optional[Source] = None
    timestamp: StrictInt = Field(..., description="Time of the event in milliseconds.")
    mode: EventMode = Field(...)
    webhook_event_id: StrictStr = Field(..., alias="webhookEventId", description="Webhook Event ID. An ID that uniquely identifies a webhook event. This is a string in ULID format.")
    delivery_context: DeliveryContext = Field(..., alias="deliveryContext")

    __properties = ["type", "source", "timestamp", "mode", "webhookEventId", "deliveryContext"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    # JSON field name that stores the object type
    __discriminator_property_name = 'type'

    # discriminator mappings
    __discriminator_value_class_map = {
        'accountLink': 'AccountLinkEvent',
        'activated': 'ActivatedEvent',
        'beacon': 'BeaconEvent',
        'botResumed': 'BotResumedEvent',
        'botSuspended': 'BotSuspendedEvent',
        'deactivated': 'DeactivatedEvent',
        'delivery': 'PnpDeliveryCompletionEvent',
        'follow': 'FollowEvent',
        'join': 'JoinEvent',
        'leave': 'LeaveEvent',
        'memberJoined': 'MemberJoinedEvent',
        'memberLeft': 'MemberLeftEvent',
        'membership': 'MembershipEvent',
        'message': 'MessageEvent',
        'module': 'ModuleEvent',
        'postback': 'PostbackEvent',
        'unfollow': 'UnfollowEvent',
        'unsend': 'UnsendEvent',
        'videoPlayComplete': 'VideoPlayCompleteEvent'
    }

    @classmethod
    def get_discriminator_value(cls, obj: dict) -> str:
        """Returns the discriminator value (object type) of the data"""
        discriminator_value = obj[cls.__discriminator_property_name]
        if discriminator_value:
            return cls.__discriminator_value_class_map.get(discriminator_value)
        else:
            return None

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Union(AccountLinkEvent, ActivatedEvent, BeaconEvent, BotResumedEvent, BotSuspendedEvent, DeactivatedEvent, FollowEvent, JoinEvent, LeaveEvent, MemberJoinedEvent, MemberLeftEvent, MembershipEvent, MessageEvent, ModuleEvent, PnpDeliveryCompletionEvent, PostbackEvent, UnfollowEvent, UnsendEvent, VideoPlayCompleteEvent):
        """Create an instance of Event from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of source
        if self.source:
            _dict['source'] = self.source.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of delivery_context
        if self.delivery_context:
            _dict['deliveryContext'] = self.delivery_context.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Union(AccountLinkEvent, ActivatedEvent, BeaconEvent, BotResumedEvent, BotSuspendedEvent, DeactivatedEvent, FollowEvent, JoinEvent, LeaveEvent, MemberJoinedEvent, MemberLeftEvent, MembershipEvent, MessageEvent, ModuleEvent, PnpDeliveryCompletionEvent, PostbackEvent, UnfollowEvent, UnsendEvent, VideoPlayCompleteEvent):
        """Create an instance of Event from a dict"""
        # look up the object type based on discriminator mapping
        object_type = cls.get_discriminator_value(obj)
        if object_type:
            klass = getattr(linebot.v3.webhooks.models, object_type)
            return klass.from_dict(obj)
        else:
            raise ValueError("Event failed to lookup discriminator value from " +
                             json.dumps(obj) + ". Discriminator property name: " + cls.__discriminator_property_name +
                             ", mapping: " + json.dumps(cls.__discriminator_value_class_map))




# coding: utf-8

# flake8: noqa

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import apis into sdk package
from linebot.v3.messaging.api.messaging_api import MessagingApi
from linebot.v3.messaging.api.messaging_api_blob import MessagingApiBlob

from linebot.v3.messaging.api.async_messaging_api import AsyncMessagingApi
from linebot.v3.messaging.api.async_messaging_api_blob import AsyncMessagingApiBlob


# import ApiClient
from linebot.v3.messaging.api_response import ApiResponse
from linebot.v3.messaging.api_client import ApiClient
from linebot.v3.messaging.async_api_client import AsyncApiClient
from linebot.v3.messaging.configuration import Configuration
from linebot.v3.messaging.exceptions import OpenApiException
from linebot.v3.messaging.exceptions import ApiTypeError
from linebot.v3.messaging.exceptions import ApiValueError
from linebot.v3.messaging.exceptions import ApiKeyError
from linebot.v3.messaging.exceptions import ApiAttributeError
from linebot.v3.messaging.exceptions import ApiException

# import models into sdk package
from linebot.v3.messaging.models.acquisition_condition_request import AcquisitionConditionRequest
from linebot.v3.messaging.models.acquisition_condition_response import AcquisitionConditionResponse
from linebot.v3.messaging.models.action import Action
from linebot.v3.messaging.models.age_demographic import AgeDemographic
from linebot.v3.messaging.models.age_demographic_filter import AgeDemographicFilter
from linebot.v3.messaging.models.all_mention_target import AllMentionTarget
from linebot.v3.messaging.models.alt_uri import AltUri
from linebot.v3.messaging.models.app_type_demographic import AppTypeDemographic
from linebot.v3.messaging.models.app_type_demographic_filter import AppTypeDemographicFilter
from linebot.v3.messaging.models.area_demographic import AreaDemographic
from linebot.v3.messaging.models.area_demographic_filter import AreaDemographicFilter
from linebot.v3.messaging.models.audience_recipient import AudienceRecipient
from linebot.v3.messaging.models.audio_message import AudioMessage
from linebot.v3.messaging.models.bot_info_response import BotInfoResponse
from linebot.v3.messaging.models.broadcast_request import BroadcastRequest
from linebot.v3.messaging.models.buttons_template import ButtonsTemplate
from linebot.v3.messaging.models.camera_action import CameraAction
from linebot.v3.messaging.models.camera_roll_action import CameraRollAction
from linebot.v3.messaging.models.carousel_column import CarouselColumn
from linebot.v3.messaging.models.carousel_template import CarouselTemplate
from linebot.v3.messaging.models.cash_back_fixed_price_info_request import CashBackFixedPriceInfoRequest
from linebot.v3.messaging.models.cash_back_fixed_price_info_response import CashBackFixedPriceInfoResponse
from linebot.v3.messaging.models.cash_back_percentage_price_info_request import CashBackPercentagePriceInfoRequest
from linebot.v3.messaging.models.cash_back_percentage_price_info_response import CashBackPercentagePriceInfoResponse
from linebot.v3.messaging.models.cash_back_price_info_request import CashBackPriceInfoRequest
from linebot.v3.messaging.models.cash_back_price_info_response import CashBackPriceInfoResponse
from linebot.v3.messaging.models.chat_reference import ChatReference
from linebot.v3.messaging.models.clipboard_action import ClipboardAction
from linebot.v3.messaging.models.clipboard_imagemap_action import ClipboardImagemapAction
from linebot.v3.messaging.models.confirm_template import ConfirmTemplate
from linebot.v3.messaging.models.coupon_cash_back_reward_request import CouponCashBackRewardRequest
from linebot.v3.messaging.models.coupon_cash_back_reward_response import CouponCashBackRewardResponse
from linebot.v3.messaging.models.coupon_create_request import CouponCreateRequest
from linebot.v3.messaging.models.coupon_create_response import CouponCreateResponse
from linebot.v3.messaging.models.coupon_discount_reward_request import CouponDiscountRewardRequest
from linebot.v3.messaging.models.coupon_discount_reward_response import CouponDiscountRewardResponse
from linebot.v3.messaging.models.coupon_free_reward_request import CouponFreeRewardRequest
from linebot.v3.messaging.models.coupon_free_reward_response import CouponFreeRewardResponse
from linebot.v3.messaging.models.coupon_gift_reward_request import CouponGiftRewardRequest
from linebot.v3.messaging.models.coupon_gift_reward_response import CouponGiftRewardResponse
from linebot.v3.messaging.models.coupon_list_response import CouponListResponse
from linebot.v3.messaging.models.coupon_message import CouponMessage
from linebot.v3.messaging.models.coupon_others_reward_request import CouponOthersRewardRequest
from linebot.v3.messaging.models.coupon_others_reward_response import CouponOthersRewardResponse
from linebot.v3.messaging.models.coupon_response import CouponResponse
from linebot.v3.messaging.models.coupon_reward_request import CouponRewardRequest
from linebot.v3.messaging.models.coupon_reward_response import CouponRewardResponse
from linebot.v3.messaging.models.create_rich_menu_alias_request import CreateRichMenuAliasRequest
from linebot.v3.messaging.models.datetime_picker_action import DatetimePickerAction
from linebot.v3.messaging.models.demographic_filter import DemographicFilter
from linebot.v3.messaging.models.discount_explicit_price_info_request import DiscountExplicitPriceInfoRequest
from linebot.v3.messaging.models.discount_explicit_price_info_response import DiscountExplicitPriceInfoResponse
from linebot.v3.messaging.models.discount_fixed_price_info_request import DiscountFixedPriceInfoRequest
from linebot.v3.messaging.models.discount_fixed_price_info_response import DiscountFixedPriceInfoResponse
from linebot.v3.messaging.models.discount_percentage_price_info_request import DiscountPercentagePriceInfoRequest
from linebot.v3.messaging.models.discount_percentage_price_info_response import DiscountPercentagePriceInfoResponse
from linebot.v3.messaging.models.discount_price_info_request import DiscountPriceInfoRequest
from linebot.v3.messaging.models.discount_price_info_response import DiscountPriceInfoResponse
from linebot.v3.messaging.models.emoji import Emoji
from linebot.v3.messaging.models.emoji_substitution_object import EmojiSubstitutionObject
from linebot.v3.messaging.models.error_detail import ErrorDetail
from linebot.v3.messaging.models.error_response import ErrorResponse
from linebot.v3.messaging.models.filter import Filter
from linebot.v3.messaging.models.flex_block_style import FlexBlockStyle
from linebot.v3.messaging.models.flex_box import FlexBox
from linebot.v3.messaging.models.flex_box_background import FlexBoxBackground
from linebot.v3.messaging.models.flex_box_border_width import FlexBoxBorderWidth
from linebot.v3.messaging.models.flex_box_corner_radius import FlexBoxCornerRadius
from linebot.v3.messaging.models.flex_box_linear_gradient import FlexBoxLinearGradient
from linebot.v3.messaging.models.flex_box_padding import FlexBoxPadding
from linebot.v3.messaging.models.flex_box_spacing import FlexBoxSpacing
from linebot.v3.messaging.models.flex_bubble import FlexBubble
from linebot.v3.messaging.models.flex_bubble_styles import FlexBubbleStyles
from linebot.v3.messaging.models.flex_button import FlexButton
from linebot.v3.messaging.models.flex_carousel import FlexCarousel
from linebot.v3.messaging.models.flex_component import FlexComponent
from linebot.v3.messaging.models.flex_container import FlexContainer
from linebot.v3.messaging.models.flex_filler import FlexFiller
from linebot.v3.messaging.models.flex_icon import FlexIcon
from linebot.v3.messaging.models.flex_icon_size import FlexIconSize
from linebot.v3.messaging.models.flex_image import FlexImage
from linebot.v3.messaging.models.flex_image_size import FlexImageSize
from linebot.v3.messaging.models.flex_margin import FlexMargin
from linebot.v3.messaging.models.flex_message import FlexMessage
from linebot.v3.messaging.models.flex_offset import FlexOffset
from linebot.v3.messaging.models.flex_separator import FlexSeparator
from linebot.v3.messaging.models.flex_span import FlexSpan
from linebot.v3.messaging.models.flex_span_size import FlexSpanSize
from linebot.v3.messaging.models.flex_text import FlexText
from linebot.v3.messaging.models.flex_text_font_size import FlexTextFontSize
from linebot.v3.messaging.models.flex_video import FlexVideo
from linebot.v3.messaging.models.gender_demographic import GenderDemographic
from linebot.v3.messaging.models.gender_demographic_filter import GenderDemographicFilter
from linebot.v3.messaging.models.get_aggregation_unit_name_list_response import GetAggregationUnitNameListResponse
from linebot.v3.messaging.models.get_aggregation_unit_usage_response import GetAggregationUnitUsageResponse
from linebot.v3.messaging.models.get_followers_response import GetFollowersResponse
from linebot.v3.messaging.models.get_joined_membership_users_response import GetJoinedMembershipUsersResponse
from linebot.v3.messaging.models.get_membership_subscription_response import GetMembershipSubscriptionResponse
from linebot.v3.messaging.models.get_message_content_transcoding_response import GetMessageContentTranscodingResponse
from linebot.v3.messaging.models.get_webhook_endpoint_response import GetWebhookEndpointResponse
from linebot.v3.messaging.models.group_member_count_response import GroupMemberCountResponse
from linebot.v3.messaging.models.group_summary_response import GroupSummaryResponse
from linebot.v3.messaging.models.group_user_profile_response import GroupUserProfileResponse
from linebot.v3.messaging.models.image_carousel_column import ImageCarouselColumn
from linebot.v3.messaging.models.image_carousel_template import ImageCarouselTemplate
from linebot.v3.messaging.models.image_message import ImageMessage
from linebot.v3.messaging.models.imagemap_action import ImagemapAction
from linebot.v3.messaging.models.imagemap_area import ImagemapArea
from linebot.v3.messaging.models.imagemap_base_size import ImagemapBaseSize
from linebot.v3.messaging.models.imagemap_external_link import ImagemapExternalLink
from linebot.v3.messaging.models.imagemap_message import ImagemapMessage
from linebot.v3.messaging.models.imagemap_video import ImagemapVideo
from linebot.v3.messaging.models.issue_link_token_response import IssueLinkTokenResponse
from linebot.v3.messaging.models.limit import Limit
from linebot.v3.messaging.models.location_action import LocationAction
from linebot.v3.messaging.models.location_message import LocationMessage
from linebot.v3.messaging.models.lottery_acquisition_condition_request import LotteryAcquisitionConditionRequest
from linebot.v3.messaging.models.lottery_acquisition_condition_response import LotteryAcquisitionConditionResponse
from linebot.v3.messaging.models.mark_messages_as_read_request import MarkMessagesAsReadRequest
from linebot.v3.messaging.models.members_ids_response import MembersIdsResponse
from linebot.v3.messaging.models.membership import Membership
from linebot.v3.messaging.models.membership_list_response import MembershipListResponse
from linebot.v3.messaging.models.mention_substitution_object import MentionSubstitutionObject
from linebot.v3.messaging.models.mention_target import MentionTarget
from linebot.v3.messaging.models.message import Message
from linebot.v3.messaging.models.message_action import MessageAction
from linebot.v3.messaging.models.message_imagemap_action import MessageImagemapAction
from linebot.v3.messaging.models.message_quota_response import MessageQuotaResponse
from linebot.v3.messaging.models.messaging_api_pager_coupon_list_response import MessagingApiPagerCouponListResponse
from linebot.v3.messaging.models.multicast_request import MulticastRequest
from linebot.v3.messaging.models.narrowcast_progress_response import NarrowcastProgressResponse
from linebot.v3.messaging.models.narrowcast_request import NarrowcastRequest
from linebot.v3.messaging.models.normal_acquisition_condition_request import NormalAcquisitionConditionRequest
from linebot.v3.messaging.models.normal_acquisition_condition_response import NormalAcquisitionConditionResponse
from linebot.v3.messaging.models.number_of_messages_response import NumberOfMessagesResponse
from linebot.v3.messaging.models.operator_demographic_filter import OperatorDemographicFilter
from linebot.v3.messaging.models.operator_recipient import OperatorRecipient
from linebot.v3.messaging.models.pnp_messages_request import PnpMessagesRequest
from linebot.v3.messaging.models.postback_action import PostbackAction
from linebot.v3.messaging.models.push_message_request import PushMessageRequest
from linebot.v3.messaging.models.push_message_response import PushMessageResponse
from linebot.v3.messaging.models.quick_reply import QuickReply
from linebot.v3.messaging.models.quick_reply_item import QuickReplyItem
from linebot.v3.messaging.models.quota_consumption_response import QuotaConsumptionResponse
from linebot.v3.messaging.models.quota_type import QuotaType
from linebot.v3.messaging.models.recipient import Recipient
from linebot.v3.messaging.models.redelivery_recipient import RedeliveryRecipient
from linebot.v3.messaging.models.referral_acquisition_condition_response import ReferralAcquisitionConditionResponse
from linebot.v3.messaging.models.reply_message_request import ReplyMessageRequest
from linebot.v3.messaging.models.reply_message_response import ReplyMessageResponse
from linebot.v3.messaging.models.rich_menu_alias_list_response import RichMenuAliasListResponse
from linebot.v3.messaging.models.rich_menu_alias_response import RichMenuAliasResponse
from linebot.v3.messaging.models.rich_menu_area import RichMenuArea
from linebot.v3.messaging.models.rich_menu_batch_link_operation import RichMenuBatchLinkOperation
from linebot.v3.messaging.models.rich_menu_batch_operation import RichMenuBatchOperation
from linebot.v3.messaging.models.rich_menu_batch_progress_phase import RichMenuBatchProgressPhase
from linebot.v3.messaging.models.rich_menu_batch_progress_response import RichMenuBatchProgressResponse
from linebot.v3.messaging.models.rich_menu_batch_request import RichMenuBatchRequest
from linebot.v3.messaging.models.rich_menu_batch_unlink_all_operation import RichMenuBatchUnlinkAllOperation
from linebot.v3.messaging.models.rich_menu_batch_unlink_operation import RichMenuBatchUnlinkOperation
from linebot.v3.messaging.models.rich_menu_bounds import RichMenuBounds
from linebot.v3.messaging.models.rich_menu_bulk_link_request import RichMenuBulkLinkRequest
from linebot.v3.messaging.models.rich_menu_bulk_unlink_request import RichMenuBulkUnlinkRequest
from linebot.v3.messaging.models.rich_menu_id_response import RichMenuIdResponse
from linebot.v3.messaging.models.rich_menu_list_response import RichMenuListResponse
from linebot.v3.messaging.models.rich_menu_request import RichMenuRequest
from linebot.v3.messaging.models.rich_menu_response import RichMenuResponse
from linebot.v3.messaging.models.rich_menu_size import RichMenuSize
from linebot.v3.messaging.models.rich_menu_switch_action import RichMenuSwitchAction
from linebot.v3.messaging.models.room_member_count_response import RoomMemberCountResponse
from linebot.v3.messaging.models.room_user_profile_response import RoomUserProfileResponse
from linebot.v3.messaging.models.sender import Sender
from linebot.v3.messaging.models.sent_message import SentMessage
from linebot.v3.messaging.models.set_webhook_endpoint_request import SetWebhookEndpointRequest
from linebot.v3.messaging.models.show_loading_animation_request import ShowLoadingAnimationRequest
from linebot.v3.messaging.models.sticker_message import StickerMessage
from linebot.v3.messaging.models.subscribed_membership_plan import SubscribedMembershipPlan
from linebot.v3.messaging.models.subscribed_membership_user import SubscribedMembershipUser
from linebot.v3.messaging.models.subscription import Subscription
from linebot.v3.messaging.models.subscription_period_demographic import SubscriptionPeriodDemographic
from linebot.v3.messaging.models.subscription_period_demographic_filter import SubscriptionPeriodDemographicFilter
from linebot.v3.messaging.models.substitution_object import SubstitutionObject
from linebot.v3.messaging.models.template import Template
from linebot.v3.messaging.models.template_image_aspect_ratio import TemplateImageAspectRatio
from linebot.v3.messaging.models.template_image_size import TemplateImageSize
from linebot.v3.messaging.models.template_message import TemplateMessage
from linebot.v3.messaging.models.test_webhook_endpoint_request import TestWebhookEndpointRequest
from linebot.v3.messaging.models.test_webhook_endpoint_response import TestWebhookEndpointResponse
from linebot.v3.messaging.models.text_message import TextMessage
from linebot.v3.messaging.models.text_message_v2 import TextMessageV2
from linebot.v3.messaging.models.uri_action import URIAction
from linebot.v3.messaging.models.uri_imagemap_action import URIImagemapAction
from linebot.v3.messaging.models.update_rich_menu_alias_request import UpdateRichMenuAliasRequest
from linebot.v3.messaging.models.user_mention_target import UserMentionTarget
from linebot.v3.messaging.models.user_profile_response import UserProfileResponse
from linebot.v3.messaging.models.validate_message_request import ValidateMessageRequest
from linebot.v3.messaging.models.video_message import VideoMessage



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import pytest
from aiohttp import web

from linebot.aiohttp_async_http_client import AiohttpAsyncHttpClient


@pytest.mark.asyncio
async def test_get(aiohttp_client):
    async def hello(request):
        return web.Response(text='Hello, world')

    app = web.Application()
    app.router.add_get('/test', hello)

    client = AiohttpAsyncHttpClient(session=await aiohttp_client(app))
    resp = await client.get('/test')
    assert resp.status_code == 200
    text = await resp.text
    assert 'Hello, world' in text


@pytest.mark.asyncio
async def test_get_json(aiohttp_client):
    async def hello(request):
        return web.json_response({'test': 'Hello, world'})

    app = web.Application()
    app.router.add_get('/test', hello)
    client = AiohttpAsyncHttpClient(session=await aiohttp_client(app))
    resp = await client.get('/test')
    assert resp.status_code == 200
    json = await resp.json
    assert 'Hello, world' == json['test']


@pytest.mark.asyncio
async def test_get_iter(aiohttp_client):
    async def hello(request):
        return web.Response(text='Hello, world')

    app = web.Application()
    app.router.add_get('/test', hello)

    client = AiohttpAsyncHttpClient(session=await aiohttp_client(app))
    resp = await client.get('/test')
    assert resp.status_code == 200

    buffer = ''
    async for data in resp.iter_content(1024):
        buffer += data.decode('utf-8')
    assert 'Hello, world' in buffer


@pytest.mark.asyncio
async def test_post(aiohttp_client):
    async def hello(request):
        return web.Response(text='Hello, world')

    app = web.Application()
    app.router.add_post('/test', hello)

    client = AiohttpAsyncHttpClient(session=await aiohttp_client(app))
    resp = await client.post('/test')
    assert resp.status_code == 200
    text = await resp.text
    assert 'Hello, world' in text


@pytest.mark.asyncio
async def test_delete(aiohttp_client):
    async def hello(request):
        return web.Response(text='Hello, world')

    app = web.Application()
    app.router.add_delete('/test', hello)

    client = AiohttpAsyncHttpClient(session=await aiohttp_client(app))
    resp = await client.delete('/test')
    assert resp.status_code == 200
    text = await resp.text
    assert 'Hello, world' in text


@pytest.mark.asyncio
async def test_put(aiohttp_client):
    async def hello(request):
        return web.Response(text='Hello, world')

    app = web.Application()
    app.router.add_put('/test', hello)

    client = AiohttpAsyncHttpClient(session=await aiohttp_client(app))
    resp = await client.put('/test')
    assert resp.status_code == 200
    text = await resp.text
    assert 'Hello, world' in text



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import sys
import unittest

import responses

from linebot import (
    LineBotApi
)

PY3 = sys.version_info[0] == 3
if PY3:
    from urllib import parse
else:
    import urlparse as parse


class TestLineBotApi(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')
        self.access_token = 'W1TeHCgfH2Liwa.....'
        self.expires_in = 2592000
        self.token_type = 'Bearer'
        self.client_assertion = 'eyJhbGciOiJSUzI.q....'
        self.client_id = 'client_id'
        self.client_secret = 'client_secret'
        self.client_assertion_type = 'urn:ietf:params:oauth:client-assertion-type:jwt-bearer'
        self.key_id = 'sDTOzw5wIfxxxxPEzcmeQA'
        self.scope = 'profile chat_message.write'

    @responses.activate
    def test_issue_channel_access_token_v2_1(self):
        endpoint = LineBotApi.DEFAULT_API_ENDPOINT + '/oauth2/v2.1/token'
        responses.add(
            responses.POST,
            endpoint,
            json={
                'access_token': self.access_token,
                'expires_in': self.expires_in,
                'token_type': self.token_type,
                'key_id': self.key_id
            },
            status=200
        )

        issue_access_token_response = self.tested.issue_channel_access_token_v2_1(
            self.client_assertion
        )

        request = responses.calls[0].request
        self.assertEqual('POST', request.method)
        self.assertEqual(endpoint, request.url)
        self.assertEqual('application/x-www-form-urlencoded', request.headers['content-type'])
        self.assertEqual(self.access_token, issue_access_token_response.access_token)
        self.assertEqual(self.expires_in, issue_access_token_response.expires_in)
        self.assertEqual(self.token_type, issue_access_token_response.token_type)

        encoded_body = parse.parse_qs(request.body)
        self.assertEqual('client_credentials', encoded_body['grant_type'][0])
        self.assertEqual(self.client_assertion_type, encoded_body['client_assertion_type'][0])
        self.assertEqual(self.client_assertion, encoded_body['client_assertion'][0])

    @responses.activate
    def test_get_channel_access_token_v2_1(self):

        endpoint = LineBotApi.DEFAULT_API_ENDPOINT + '/oauth2/v2.1/tokens'
        responses.add(
            responses.GET,
            endpoint,
            json={
                'access_tokens': [
                    'fgIkeLcl3.....',
                    'eyJhbGciO.....',
                    'oeLklsSi7.....'
                ]
            },
            status=200
        )
        channel_access_tokens_response = self.tested.get_channel_access_tokens_v2_1(
            self.client_assertion
        )

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            parse.unquote(request.url),
            parse.unquote('{}?client_assertion={}&client_assertion_type={}'.format(
                endpoint, self.client_assertion, self.client_assertion_type
            ))
        )
        self.assertEqual(channel_access_tokens_response.access_tokens, [
            'fgIkeLcl3.....',
            'eyJhbGciO.....',
            'oeLklsSi7.....'
        ])

    @responses.activate
    def test_revoke_channel_access_token_v2_1(self):
        endpoint = LineBotApi.DEFAULT_API_ENDPOINT + '/oauth2/v2.1/revoke'

        responses.add(
            responses.POST,
            endpoint,
            status=200
        )

        self.tested.revoke_channel_access_token_v2_1(
            self.client_id, self.client_secret, self.access_token
        )

        request = responses.calls[0].request
        self.assertEqual('POST', request.method)
        self.assertEqual(endpoint, request.url)

        encoded_body = parse.parse_qs(request.body)
        self.assertEqual(self.client_id, encoded_body['client_id'][0])
        self.assertEqual(self.client_secret, encoded_body['client_secret'][0])
        self.assertEqual(self.access_token, encoded_body['access_token'][0])

    @responses.activate
    def test_verify_channel_access_token_v2_1(self):
        endpoint = LineBotApi.DEFAULT_API_ENDPOINT + '/oauth2/v2.1/verify'

        responses.add(
            responses.GET,
            endpoint,
            status=200,
            json={
                'client_id': self.client_id,
                'expires_in': self.expires_in,
                'scope': self.scope,
            },
        )

        self.tested.verify_channel_access_token_v2_1(self.access_token)

        request = responses.calls[0].request
        self.assertEqual('GET', request.method)
        self.assertEqual(
            parse.unquote(request.url),
            parse.unquote('{}?access_token={}'.format(
                endpoint, self.access_token
            ))
        )

    @responses.activate
    def test_get_channel_token_key_ids_v2_1(self):
        endpoint = LineBotApi.DEFAULT_API_ENDPOINT + '/oauth2/v2.1/tokens/kid'

        responses.add(
            responses.GET,
            endpoint,
            status=200,
            json={
                'kids': [self.key_id],
            },
        )

        self.tested.get_channel_token_key_ids_v2_1(self.client_assertion, self.client_assertion_type)

        request = responses.calls[0].request
        self.assertEqual('GET', request.method)
        self.assertEqual(
            parse.unquote(request.url),
            parse.unquote('{}?client_assertion={}&client_assertion_type={}'.format(
                endpoint, self.client_assertion, self.client_assertion_type
            ))
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List
from pydantic.v1 import BaseModel, Field, conlist, constr, validator
from linebot.v3.webhooks.models.event import Event

class CallbackRequest(BaseModel):
    """
    The request body contains a JSON object with the user ID of a bot that should receive webhook events and an array of webhook event objects. 
    https://developers.line.biz/en/reference/messaging-api/#request-body
    """
    destination: constr(strict=True, max_length=33, min_length=33) = Field(..., description="User ID of a bot that should receive webhook events. The user ID value is a string that matches the regular expression, `U[0-9a-f]{32}`. ")
    events: conlist(Event) = Field(..., description="Array of webhook event objects. The LINE Platform may send an empty array that doesn't include a webhook event object to confirm communication. ")

    __properties = ["destination", "events"]

    @validator('destination')
    def destination_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^U[0-9a-f]{32}$", value):
            raise ValueError(r"must validate the regular expression /^U[0-9a-f]{32}$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CallbackRequest:
        """Create an instance of CallbackRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in events (list)
        _items = []
        if self.events:
            for _item in self.events:
                if _item:
                    _items.append(_item.to_dict())
            _dict['events'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CallbackRequest:
        """Create an instance of CallbackRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CallbackRequest.parse_obj(obj)

        _obj = CallbackRequest.parse_obj({
            "destination": obj.get("destination"),
            "events": [Event.from_dict(_item) for _item in obj.get("events")] if obj.get("events") is not None else None
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

import responses

from linebot import (
    LineBotApi
)
from linebot.models import (
    GenderInsight, AreaInsight, AgeInsight, AppTypeInsight, SubscriptionPeriodInsight
)


class TestLineBotApi(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')
        self.date = '20190101'
        self.request_id = 'f70dd685-499a-4231-a441-f24b8d4fba21'

    @responses.activate
    def test_get_insight_message_delivery(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/insight/message/delivery?date={date}'.format(date=self.date),
            json={
                'status': 'ready',
                'broadcast': 100,
                'targeting': 200,
                'autoResponse': 300,
                'welcomeResponse': 400,
                'chat': 500,
                'apiBroadcast': 600,
                'apiPush': 700,
                'apiMulticast': 800,
            },
            status=200
        )

        res = self.tested.get_insight_message_delivery(self.date)
        request = responses.calls[0].request
        self.assertEqual('GET', request.method)
        self.assertEqual('ready', res.status)
        self.assertEqual(100, res.broadcast)
        self.assertEqual(200, res.targeting)
        self.assertEqual(300, res.auto_response)
        self.assertEqual(400, res.welcome_response)
        self.assertEqual(500, res.chat)
        self.assertEqual(600, res.api_broadcast)
        self.assertEqual(700, res.api_push)
        self.assertEqual(800, res.api_multicast)
        self.assertEqual(None, res.api_reply)

    @responses.activate
    def test_get_insight_followers(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/insight/followers?date={date}'.format(date=self.date),
            json={
                'status': 'ready',
                'followers': 100,
                'targetedReaches': 200,
                'blocks': 300
            },
            status=200
        )

        res = self.tested.get_insight_followers(self.date)
        request = responses.calls[0].request
        self.assertEqual('GET', request.method)
        self.assertEqual('ready', res.status)
        self.assertEqual(100, res.followers)
        self.assertEqual(200, res.targeted_reaches)
        self.assertEqual(300, res.blocks)

    @responses.activate
    def test_get_insight_demographic(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/insight/demographic',
            json={
                'available': True,
                'genders': [
                    {
                        'gender': 'unknown',
                        'percentage': 37.6
                    },
                    {
                        'gender': 'male',
                        'percentage': 31.8
                    },
                    {
                        'gender': 'female',
                        'percentage': 30.6
                    }
                ],
                'ages': [
                    {
                        'age': 'unknown',
                        'percentage': 37.6
                    },
                    {
                        'age': 'from50',
                        'percentage': 17.3
                    },
                ],
                'areas': [
                    {
                        'area': 'unknown',
                        'percentage': 50.5
                    },
                    {
                        'area': 'Tokyo',
                        'percentage': 49.5
                    },
                ],
                'appTypes': [
                    {
                        'appType': 'ios',
                        'percentage': 62.4
                    },
                    {
                        'appType': 'android',
                        'percentage': 27.7
                    },
                    {
                        'appType': 'others',
                        'percentage': 9.9
                    }
                ],
                'subscriptionPeriods': [
                    {
                        'subscriptionPeriod': 'over365days',
                        'percentage': 96.4
                    },
                    {
                        'subscriptionPeriod': 'within365days',
                        'percentage': 1.9
                    },
                    {
                        'subscriptionPeriod': 'within180days',
                        'percentage': 1.2
                    },
                    {
                        'subscriptionPeriod': 'within90days',
                        'percentage': 0.5
                    },
                    {
                        'subscriptionPeriod': 'within30days',
                        'percentage': 0.1
                    },
                    {
                        'subscriptionPeriod': 'within7days',
                        'percentage': 0
                    }
                ]
            },
            status=200
        )

        res = self.tested.get_insight_demographic()
        request = responses.calls[0].request
        self.assertEqual('GET', request.method)
        self.assertEqual(True, res.available)
        self.assertIn(GenderInsight(gender='male', percentage=31.8), res.genders)
        self.assertIn(AgeInsight(age='from50', percentage=17.3), res.ages)
        self.assertIn(AreaInsight(area='Tokyo', percentage=49.5), res.areas)
        self.assertIn(AppTypeInsight(app_type='ios', percentage=62.4), res.app_types)
        self.assertIn(
            SubscriptionPeriodInsight(subscription_period='over365days', percentage=96.4),
            res.subscription_periods
        )

    @responses.activate
    def test_get_insight_message_event(self):
        json = {
            'overview': {
                'requestId': 'f70dd685-499a-4231-a441-f24b8d4fba21',
                'timestamp': 1568214000,
                'delivered': 32,
                'uniqueImpression': 4,
                'uniqueClick': None,
                'uniqueMediaPlayed': 2,
                'uniqueMediaPlayed100Percent': -1
            },
            'messages': [
                {
                    'seq': 1,
                    'impression': 18,
                    'mediaPlayed': 11,
                    'mediaPlayed25Percent': -1,
                    'mediaPlayed50Percent': -1,
                    'mediaPlayed75Percent': -1,
                    'mediaPlayed100Percent': -1,
                    'uniqueMediaPlayed': 2,
                    'uniqueMediaPlayed25Percent': -1,
                    'uniqueMediaPlayed50Percent': -1,
                    'uniqueMediaPlayed75Percent': -1,
                    'uniqueMediaPlayed100Percent': -1
                }
            ],
            'clicks': [
                {
                    'seq': 1,
                    'url': 'https://www.yahoo.co.jp/',
                    'click': -1,
                    'uniqueClick': -1,
                    'uniqueClickOfRequest': -1
                },
                {
                    'seq': 1,
                    'url': 'https://www.google.com/?hl=ja',
                    'click': -1,
                    'uniqueClick': -1,
                    'uniqueClickOfRequest': -1
                }
            ]
        }
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/insight/message/event?requestId={request_id}'.format(request_id=self.request_id),
            status=200,
            json=json,
        )

        res = self.tested.get_insight_message_event(self.request_id)
        request = responses.calls[0].request
        self.assertEqual('GET', request.method)
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/insight/message/event?requestId={request_id}'.format(request_id=self.request_id)
        )

        self.assertEqual(res.overview.timestamp, json['overview']['timestamp'])
        self.assertEqual(res.overview.unique_media_played_100_percent, json['overview']['uniqueMediaPlayed100Percent'])
        self.assertEqual(res.messages[0].seq, json['messages'][0]['seq'])
        self.assertEqual(res.messages[0].media_played_50_percent, json['messages'][0]['mediaPlayed50Percent'])
        self.assertEqual(res.clicks[0].url, json['clicks'][0]['url'])
        self.assertEqual(res.clicks[0].unique_click_of_request, json['clicks'][0]['uniqueClickOfRequest'])

    @responses.activate
    def test_get_statistics_per_unit(self):
        json = {
            'overview': {
                "uniqueImpression": 40,
                "uniqueClick": 30,
                "uniqueMediaPlayed": 25,
                "uniqueMediaPlayed100Percent": None
            },
            'messages': [
                {
                    'seq': 1,
                    'impression': 42,
                    'mediaPlayed': 30,
                    'mediaPlayed25Percent': None,
                    'mediaPlayed50Percent': None,
                    'mediaPlayed75Percent': None,
                    'mediaPlayed100Percent': None,
                    'uniqueMediaPlayed': 25,
                    'uniqueMediaPlayed25Percent': None,
                    'uniqueMediaPlayed50Percent': None,
                    'uniqueMediaPlayed75Percent': None,
                    'uniqueMediaPlayed100Percent': None
                }
            ],
            'clicks': [
                {
                    'seq': 1,
                    'url': 'https://developers.line.biz/',
                    'click': 35,
                    'uniqueClick': 25,
                    'uniqueClickOfRequest': None
                },
                {
                    'seq': 1,
                    'url': 'https://developers.line.biz/',
                    'click': 20,
                    'uniqueClick': None,
                    'uniqueClickOfRequest': None
                }
            ]
        }
        request_param = {
            'custom_aggregation_unit': 'promotion_a',
            'from': '20210301',
            'to': '20210331'
        }
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/insight/message/event/aggregation'
            '?customAggregationUnit={custom_aggregation_unit}'
            '&from={from_date}&to={to_date}'.format(
                custom_aggregation_unit=request_param['custom_aggregation_unit'],
                from_date=request_param['from'], to_date=request_param['to']),
            status=200,
            json=json,
        )

        res = self.tested.get_statistics_per_unit(
            request_param['custom_aggregation_unit'], request_param['from'], request_param['to'])
        request = responses.calls[0].request
        self.assertEqual('GET', request.method)
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/insight/message/event/aggregation'
            '?customAggregationUnit={custom_aggregation_unit}'
            '&from={from_date}&to={to_date}'.format(
                custom_aggregation_unit=request_param['custom_aggregation_unit'],
                from_date=request_param['from'], to_date=request_param['to'])
        )

        self.assertEqual(res.overview.unique_media_played_100_percent, json['overview']['uniqueMediaPlayed100Percent'])
        self.assertEqual(res.messages[0].seq, json['messages'][0]['seq'])
        self.assertEqual(res.messages[0].media_played_50_percent, json['messages'][0]['mediaPlayed50Percent'])
        self.assertEqual(res.clicks[0].url, json['clicks'][0]['url'])
        self.assertEqual(res.clicks[0].unique_click_of_request, json['clicks'][0]['uniqueClickOfRequest'])


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

# flake8: noqa

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import apis into sdk package
from linebot.v3.module.api.line_module import LineModule

from linebot.v3.module.api.async_line_module import AsyncLineModule


# import ApiClient
from linebot.v3.module.api_response import ApiResponse
from linebot.v3.module.api_client import ApiClient
from linebot.v3.module.async_api_client import AsyncApiClient
from linebot.v3.module.configuration import Configuration
from linebot.v3.module.exceptions import OpenApiException
from linebot.v3.module.exceptions import ApiTypeError
from linebot.v3.module.exceptions import ApiValueError
from linebot.v3.module.exceptions import ApiKeyError
from linebot.v3.module.exceptions import ApiAttributeError
from linebot.v3.module.exceptions import ApiException

# import models into sdk package
from linebot.v3.module.models.acquire_chat_control_request import AcquireChatControlRequest
from linebot.v3.module.models.detach_module_request import DetachModuleRequest
from linebot.v3.module.models.get_modules_response import GetModulesResponse
from linebot.v3.module.models.module_bot import ModuleBot



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.base module."""


import json

from .. import utils


class Base(object):
    """Base class of model.

    Suitable for JSON base data.
    """

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        pass

    def __str__(self):
        """__str__ method."""
        return self.as_json_string()

    def __repr__(self):
        """__repr__ method."""
        return str(self)

    def __eq__(self, other):
        """__eq__ method.

        :param other:
        """
        return other and self.as_json_dict() == other.as_json_dict()

    def __ne__(self, other):
        """__ne__ method.

        :param other:
        """
        return not self.__eq__(other)

    def as_json_string(self):
        """Return JSON string from this object.

        :rtype: str
        """
        return json.dumps(self.as_json_dict(), sort_keys=True)

    def as_json_dict(self):
        """Return dictionary from this object.

        :return: dict
        """
        data = {}
        for key, value in self.__dict__.items():
            camel_key = utils.to_camel_case(key)
            if isinstance(value, (list, tuple, set)):
                data[camel_key] = list()
                for item in value:
                    if hasattr(item, 'as_json_dict'):
                        data[camel_key].append(item.as_json_dict())
                    else:
                        data[camel_key].append(item)

            elif hasattr(value, 'as_json_dict'):
                data[camel_key] = value.as_json_dict()
            elif value is not None:
                data[camel_key] = value

        return data

    @classmethod
    def new_from_json_dict(cls, data, use_raw_message=False):
        """Create a new instance from a dict.

        :param data: JSON dict
        :param bool use_raw_message: Using original Message key as attribute
        """
        if use_raw_message:
            return cls(use_raw_message=use_raw_message, **data)

        new_data = {utils.to_snake_case(key): value
                    for key, value in data.items()}

        return cls(**new_data)

    @staticmethod
    def get_or_new_from_json_dict(data, cls):
        """Get `cls` object w/ deserialization from json if needed.

        If data is instance of cls, return data.
        Else if data is instance of dict, create instance from dict.
        Else, return None.

        :param data:
        :param cls:
        :rtype: object
        """
        if isinstance(data, cls):
            return data
        elif isinstance(data, dict):
            return cls.new_from_json_dict(data)

        return None

    @staticmethod
    def get_or_new_from_json_dict_with_types(
            data, cls_map, type_key='type', use_raw_message=False
    ):
        """Get `cls` object w/ deserialization from json by using type key hint if needed.

        If data is instance of one of cls, return data.
        Else if data is instance of dict, create instance from dict.
        Else, return None.

        :param data:
        :param cls_map:
        :param type_key:
        :rtype: object
        :param bool use_raw_message: Using original Message key as attribute
        """
        if isinstance(data, tuple(cls_map.values())):
            return data
        elif isinstance(data, dict):
            type_val = data[type_key]
            if type_val in cls_map:
                return cls_map[type_val].new_from_json_dict(data, use_raw_message=use_raw_message)

        return None
